﻿using System;
using MVVM;
using DataGridExecutionList.Model;
using System.Collections.ObjectModel;
using DataGridExecutionList.Data;
using System.Windows.Input;
using System.ComponentModel;
using System.Threading;
using System.Collections.Specialized;

namespace DataGridExecutionList.ViewModel
{
    class RunningCommandsViewModel : ViewModelBase
    {
        #region Member Variables
        private RelayCommand _StartExecutionCommand;
        #endregion

        #region Constructors
        /// <summary>
        /// The default constructor
        /// </summary>
        public RunningCommandsViewModel()
        {
        }
        #endregion

        #region Properties
        public BindingList<Command> Commands
        {
            get
            {
                if (_Commands == null)
                {
                    _Commands = new BindingList<Command>();
                    foreach (var command in Database.Commands)
                    {
                        _Commands.Add(command);
                    }
                }
                return _Commands;
            }
        } private BindingList<Command> _Commands;

        public Command CurrentCommand
        {
            get { return Commands[Database.CurrentCommand]; }
        }

        public ICommand StartExecutionCommand
        {
            get
            {
                if (_StartExecutionCommand == null)
                    _StartExecutionCommand = new RelayCommand(param => ExecuteNextCommand());
                return _StartExecutionCommand;
            }
            set { _StartExecutionCommand = value as RelayCommand; }
        }
        #endregion

        #region Functions
        private void ExecuteNextCommand()
        {
            // Mark the command as in progress
            Database.Commands[Database.CurrentCommand].State = Command.ExecutionState.InProgress;

            // Run the command
            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted);
            worker.RunWorkerAsync();
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Simulate doing work.
            // Really we would load a new process and launch this but for now....
            Thread.Sleep(1000);
            // Just set the command to done
            Database.Commands[Database.CurrentCommand].State = Command.ExecutionState.Success;
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Increment the current command            
            Database.CurrentCommand++;

            if (Database.CurrentCommand < Database.Commands.Count)
            {
                ExecuteNextCommand();
                NotifyPropertyChanged("CurrentCommand");
            }
        }
        #endregion

        #region Enums
        #endregion
    }
}
